#!/usr/bin/env python2
# -*- coding: utf-8 -*-

# This is a much simplified Python translation of a Script-Fu plugin by
# Hans Schopmeijer. It is said that the most pleasing bokeh effects
# result when the discs are uniformly coloured and circular. So that
# works out quite nicely. The discs are combined additively on the bokeh
# layer to vaguely emulate the photons better.

from gimpfu import *


def bokeh(timg, tdraw, radius, lightness, saturation, blur):
    width = pdb.gimp_image_width(timg)
    height = pdb.gimp_image_height(timg)
    diameter = 2 * radius
    vectors = pdb.gimp_image_get_active_vectors(timg)
    
    # assume our path is a single stroke
    # with only straight edges
    gpoints = pdb.gimp_vectors_stroke_get_points(vectors, 1)
    points = gpoints[2]

    # don't make a mess of the undo history    
    pdb.gimp_image_undo_group_start(timg)
    pdb.gimp_context_push()
    
    # recognise our layers so that we can keep things tidy across
    # multiple bokeh applications
    blur_layer = pdb.gimp_image_get_layer_by_name(timg,'blur')
    if blur_layer is None:
        blur_layer = pdb.gimp_layer_copy(timg.layers[0],1)
        pdb.gimp_image_insert_layer(timg, blur_layer, None, 0)
        pdb.gimp_item_set_name(blur_layer,"blur")
    
    bokeh_layer = pdb.gimp_image_get_layer_by_name(timg,'bokeh')
    if bokeh_layer is None:
        bokeh_layer = pdb.gimp_layer_new(timg, width, height, RGBA_IMAGE, "bokeh", 100, NORMAL_MODE)
        pdb.gimp_image_insert_layer(timg, bokeh_layer, None, 0)
    
    for j in range(0,gpoints[1],4):
        x = points[j]
        y = points[j + 1]        
        color = pdb.gimp_image_pick_color(timg, tdraw, x, y, False, True, 10)

        #a non-destructive black circle since addition mode won't draw on transparency
        pdb.gimp_image_select_ellipse(timg, CHANNEL_OP_REPLACE, x - radius, y - radius, diameter, diameter)
        pdb.gimp_selection_feather(timg, 2)
        pdb.gimp_context_set_foreground('#000000')
        pdb.gimp_edit_bucket_fill_full(bokeh_layer, 0,BEHIND_MODE,100,0,False,True,0,0,0)
        
        #and then our circle
        pdb.gimp_context_set_foreground(color)
        pdb.gimp_edit_bucket_fill_full(bokeh_layer, 0,ADDITION_MODE,100,0,False,True,0,0,0)
        
    pdb.gimp_selection_none(timg)
    if blur > 0:
        pdb.plug_in_gauss_iir2(timg, blur_layer, blur, blur)
    pdb.gimp_hue_saturation(bokeh_layer, 0, 0, lightness, saturation)
    pdb.gimp_layer_set_mode(bokeh_layer, SCREEN_MODE)
    
    pdb.gimp_displays_flush()
    pdb.gimp_image_undo_group_end(timg)
    pdb.gimp_context_pop()
    pdb.gimp_item_set_visible(vectors,False)
    pdb.gimp_image_set_active_layer(timg, tdraw)

    
    
register(
    "python_fu_bokeh",
    "Very Basic Bokeh",
    "Creates a Bokeh-effect from chosen lightsources in the active layer.",
    "Hans Schopmeijer/Jonni Bidwell",
    "GNU General Public License",
    "11-06-2014",
    "<Image>/Filters/My Filters/PyBokeh...",
    "RGB* GRAY*",
    [
    (PF_ADJUSTMENT,  "radius",  "Bokeh-disc radius (px)", 40, (1,100,1)),
    (PF_ADJUSTMENT,  "lightness", "lightness correction bokehlayer",  50, (-100, 100, 1)),
    (PF_ADJUSTMENT,  "saturation", "Saturation correction bokehlayer",90, (-100, 100, 1)),
    (PF_ADJUSTMENT, "blur", "Blur-radius of the inserted layer(px)", 50, (0, 200, 1))
    ],
    [],
    bokeh
    )

main()
